<?php
/*
 * Plugin Name: My WP Glossary
 * Plugin URI: https://whodunit.fr/my-wp-glossary
 * Description: A glossary block for your WordPress website, with structured data and powered by a Gutenberg block or a shortcode.
 * Version: 0.5
 * Requires at least: 5.0
 * Requires PHP: 5.6
 * Tested up to: 5.6
 * Author: Whodunit
 * Author URI: https://whodunit.fr
 * Contributors: whodunitagency, alexischenal, audrasjb, leprincenoir, virginienacci
 * License: GPLv2
 * License URI: http://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: my-wp-glossary
 */

if ( ! defined( 'ABSPATH' ) ) {
	die( 'Invalid request.' );
}

// Enqueue assets
function mywpglossary_enqueue_styles_public() {
	wp_enqueue_style(
		'mywpglossary-public-styles',
		plugin_dir_url( __FILE__ ) . 'css/public.css',
		array(),
		'',
		'all'
	);
	wp_enqueue_script(
		'mywpglossary-public-scripts',
		plugin_dir_url( __FILE__ ) . 'js/public.js',
		array( 'jquery' ),
		'',
		true
	);
	wp_localize_script( 'mywpglossary-public-scripts', 'exclude_tag', apply_filters( 'override_exclude_tag', 'a,h1,h2,h3,h4,h5,h6' ) );
}
add_action( 'wp_enqueue_scripts', 'mywpglossary_enqueue_styles_public' );


// Create CPT
function mywpglossary_register_post_type() {
	$labels = array(
		'name'          => esc_html__( 'Definitions', 'my-wp-glossary' ),
		'singular_name' => esc_html__( 'Definition', 'my-wp-glossary' ),
		'menu_name'     => esc_html__( 'Glossary', 'my-wp-glossary' ),
		'add_new'       => esc_html__( 'Add new', 'my-wp-glossary' ),
		'add_new_item'  => esc_html__( 'Add new definition', 'my-wp-glossary' ),
		'new_item'      => esc_html__( 'New definition', 'my-wp-glossary' ),
		'edit_item'     => esc_html__( 'Edit definition', 'my-wp-glossary' ),
		'view_item'     => esc_html__( 'View definition', 'my-wp-glossary' ),
		'all_items'     => esc_html__( 'All definitions', 'my-wp-glossary' ),
		'search_items'  => esc_html__( 'Search definitions', 'my-wp-glossary' ),
	);
	$args = array(
		'labels'              => $labels,
		'public'              => true,
		'publicly_queryable'  => true,
		'show_ui'             => true,
		'show_in_menu'        => true,
		'show_in_nav_menu'    => true,
		'show_in_rest'        => true,
		'query_var'           => true,
		'rewrite'             => array( 'slug' => 'glossary' ),
		'capability_type'     => 'page',
		'has_archive'         => false,
		'exclude_from_search' => true,
		'hierarchical'        => false,
		'menu_icon'           => 'dashicons-editor-textcolor',
		'supports'            => array( 'title', 'editor', 'custom-fields' )
	);
	register_post_type( 'mywpglossary', $args );
}
add_action( 'init', 'mywpglossary_register_post_type' );

// CPT Order on admin
function mywpglossary_list_screen_posts_order( $query ) {
	if ( ! is_admin() || null == get_current_screen() ) {
		return;
	}
	$current_screen = get_current_screen();
	if ( 'mywpglossary' === $current_screen->post_type ) {
		$query->set( 'orderby', 'meta_value title' );
		$query->set( 'meta_key', 'mywpglossary_letter' );
		$query->set( 'order', 'ASC' );
	}
}
add_action( 'pre_get_posts', 'mywpglossary_list_screen_posts_order' );

function mywpglossary_list_screen_add_column( $columns ) {
	$columns = array(
		'cb' => '<input type="checkbox" />',
		'title' => esc_html__( 'Glossary definitions', 'my-wp-glossary' ),
		'mywpglossary-letter' => esc_html__( 'Letter', 'my-wp-glossary' ),
		'mywpglossary-date-modified' => esc_html__( 'Last modified', 'my-wp-glossary' ),
	);
	return $columns;
}
add_filter( 'manage_mywpglossary_posts_columns', 'mywpglossary_list_screen_add_column' );

function mywpglossary_list_screen_fill_column( $column, $ID ) {
	global $post;
	switch( $column ) {
		case 'mywpglossary-letter' :
			if ( get_post_meta( $post->ID, 'mywpglossary_letter', true ) ) {
				echo get_post_meta( $post->ID, 'mywpglossary_letter', true );
			} else {
				esc_html_e( 'Yet undefined.', 'my-wp-glossary' );
			}
			break;
		case 'mywpglossary-date-modified' :
			$d = get_date_from_gmt( $post->post_modified, 'Y-m-d H:i:s' );
			echo sprintf(
			/* translators: %1$s: Date the definition was last modified. %2$s: Time the definition was last modified. */
				esc_html__( '%1$s at %2$s', 'my-wp-glossary' ),
				date_i18n( get_option( 'date_format' ), strtotime( $d ) ),
				date_i18n( get_option( 'time_format' ), strtotime( $d ) )
			);
			if ( get_post_meta( $ID, '_edit_last', true ) ) {
				$last_user = get_userdata( get_post_meta( $ID, '_edit_last', true ) );
				echo ' ' . esc_html__( 'by', 'my-wp-glossary' ) . ' ' . $last_user->display_name;
			}
			break;
		default :
			break;
	}
}
add_action( 'manage_mywpglossary_posts_custom_column' , 'mywpglossary_list_screen_fill_column', 10, 2 );


function mywpglossary_add_meta_box() {
	add_meta_box(
		'mywpglossary_section',
		__( 'Definition’s letter', 'my-wp-glossary' ),
		'mywpglossary_meta_box_callback',
		'mywpglossary',
		'side',
		'high'
	);
}
add_action( 'add_meta_boxes', 'mywpglossary_add_meta_box' );

function mywpglossary_meta_box_callback( $object ) {
	wp_nonce_field( basename( __FILE__ ), 'meta-box-nonce' );
	$letter = '';
	if ( get_post_meta( $object->ID, 'mywpglossary_letter', true ) ) {
		$letter = get_post_meta( $object->ID, 'mywpglossary_letter', true );
	}
	?>
	<p>
		<label for="meta-box-departement"><?php _e( 'Select a letter', 'my-wp-glossary' ); ?></label>
	</p>
	<?php $alphas = range( 'A', 'Z' ); ?>
	<p>
		<select name="meta_box_mywpglossary_letter" id="meta_box_mywpglossary_letter">
			<option value=""><?php _e( '— Select —', 'my-wp-glossary' ); ?></option>
			<?php foreach ( $alphas as $alpha ) : ?>
				<option value="<?php echo $alpha; ?>" <?php if ( $letter === $alpha ) { echo 'selected'; } ?>>
					<?php echo $alpha; ?>
				</option>
			<?php endforeach; ?>
		</select>
	</p>
	<?php
}

function mywpglossary_save_metaboxes( $post_id, $post, $update ) {
	if ( ! isset( $_POST['meta-box-nonce'] ) || ! wp_verify_nonce( $_POST['meta-box-nonce'], basename( __FILE__ ) ) ) {
		return $post_id;
	}
	if( ! current_user_can( 'edit_post', $post_id ) ) {
		return $post_id;
	}
	if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
		return $post_id;
	}
	if ( 'mywpglossary' !== $post->post_type ) {
		return $post_id;
	}

	$letter = '';
	if( isset( $_POST['meta_box_mywpglossary_letter'] ) ) {
		$letter = esc_html( $_POST['meta_box_mywpglossary_letter'] );
		update_post_meta( $post_id, 'mywpglossary_letter', $letter );
	}
	mywpglossary_get_transient_glossary( true );
}
add_action( 'save_post', 'mywpglossary_save_metaboxes', 10, 3 );

function mywpglossary_template_render() {
	$args = array(
		'post_type'      => 'mywpglossary',
		'posts_per_page' => -1,
	);
	$query_glossary = new WP_Query( $args );
	if ( $query_glossary->have_posts() ) {
		$glossary = array();
		while ( $query_glossary->have_posts() ) {
			$query_glossary->the_post();
			global $post;
			if ( get_post_meta( get_the_id(), 'mywpglossary_letter', true ) ) {
				$letter = get_post_meta( get_the_id(), 'mywpglossary_letter', true );
			}
			if ( empty( $letter ) || empty( $post->post_content ) ) {
				continue;
			}
			$glossary[ $letter ][ $post->post_name ] = array(
				'title' => get_the_title(),
				'content' => get_the_content(),
			);
		}
		ksort( $glossary );

		// Wrapper
		$html = '<div class="mywpglossary">';
		// Navigation
		$html  .= '<div class="mywpglossary-letters">';
		$alphas = range( 'A', 'Z' );
		foreach ( $alphas as $alpha ) {
			if ( isset( $glossary[ $alpha ] ) ) {
				$html .= '<a href="#mywpglossary-letter-' . $alpha . '" class="mywpglossary-letter">' . $alpha . '</a>';
			} else {
				$html .= '<span class="mywpglossary-letter inactive">' . $alpha . '</span>';
			}
		}
		$html .= '</div>';

		// List
		$html .= '<div itemscope itemtype="https://schema.org/DefinedTermSet" class="mywpglossary-list">';
		foreach ( $glossary as $letter => $entry ) {
			$html .= '<div id="mywpglossary-letter-' . $letter . '" class="mywpglossary-list-letter">' . $letter . '</div>';
			$html .= '<div class="mywpglossary-letter-content">';
			foreach ( $entry as $slug => $entry_data ) {
				$html .= '<div itemscope itemtype="https://schema.org/DefinedTerm" class="mywpglossary-list-entry">';
				$html .= '<div itemprop="name" id="mywpglossary-term-' . $slug . '" class="mywpglossary-list-entry-title">' . $entry_data['title'] . '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M7.41 8.59L12 13.17l4.59-4.58L18 10l-6 6-6-6 1.41-1.41z"/></svg>' . '</div>';
				$html .= '<div itemprop="description" class="mywpglossary-list-entry-description" data-glossary="mywpglossary-term-' . $slug . '">';
				$html .= $entry_data['content'];
				$html .= '</div>';
				$html .= '</div>';
			}
			$html .= '</div>';
		}

		// FIn Liste
		$html .= '</div>';

		// Fin Wrapper
		$html .= '</div>';

		return $html;
	}
	wp_reset_postdata();
}

// Shortcode declaration
function mywpglossary_init_shortcode() {
	function mywpglossary_shortcode() {
		$html = mywpglossary_template_render();
		return $html;
	}
	add_shortcode( 'glossary', 'mywpglossary_shortcode' );
}
add_action( 'init', 'mywpglossary_init_shortcode' );

// Block declaration
function mywpglossary_block_render( $attributes, $content ) {
	$content = mywpglossary_template_render();
	return $content;
}

// Single Definitions redirect
function mywpglossary_redirect_glossary_single() {
	if ( ! is_singular( 'mywpglossary' ) ) {
		return;
	}
	global $post;
	$post_slug = $post->post_name;
	$link = mywpglossary_get_template_id();
	wp_redirect( $link . '#' . $post_slug, 301 );
	exit;
}
add_action( 'template_redirect', 'mywpglossary_redirect_glossary_single' );

function mywpglossary_init_block() {
	if ( function_exists( 'register_block_type' ) ) {
		wp_register_script(
			'mywpglossary-block',
			plugins_url( 'js/block.js', __FILE__ ),
			array( 'wp-blocks', 'wp-i18n', 'wp-element', 'wp-components' )
		);

		register_block_type(
			'mywpglossary/glossary',
			array(
				'editor_script'   => 'mywpglossary-block',
				'render_callback' => 'mywpglossary_block_render',
				'attributes'      => array(),
			)
		);
	}
}
add_action( 'init', 'mywpglossary_init_block' );

// Get the first page used for template
function mywpglossary_get_template_id() {
	global $wpdb;

	$query_shortcode = "SELECT ID, post_title FROM " . $wpdb->posts . " WHERE post_content LIKE '%<!-- wp:mywpglossary/glossary /-->%' AND post_status = 'publish'";
	$results_shortcode = $wpdb->get_row( $wpdb->prepare( $query_shortcode ) );
	$link = apply_filters( 'mywpglossary_override_glossary_link', get_the_permalink( $results_shortcode->ID ) );

	return $link;
}

// Perform PHP matching on the_content
function mywpglossary_get_transient_glossary( $forceUpdate = false ) {
	$name = 'transient_glossary';

	if ( $forceUpdate || false == $result = get_transient( $name ) ) {

		$link = mywpglossary_get_template_id();

		$args = array(
			'post_type'      => 'mywpglossary',
			'posts_per_page' => -1,
		);
		$query_glossary = new WP_Query( $args );
		$glossary = array();
		$glossary_search = array();
		if ( $query_glossary->have_posts() ) {
			while ( $query_glossary->have_posts() ) {
				$query_glossary->the_post();
				$glossary_search[sanitize_title(get_the_title())] = get_the_title();
				$glossary[sanitize_title(get_the_title())] = array(
					'begin' =>	'<span class="mywpglossary-term-def" data-url="' . $link . '#' . sanitize_title( get_the_title() ) . '">',
					'end' =>	'</span>'
				);
			}
		}
		wp_reset_postdata();

		set_transient( $name, json_encode( array( 'glossary_search' => $glossary_search, 'glossary' => $glossary ) ), 21600 ); // 6 heures de cache
		$result = json_encode( array( 'glossary_search' => $glossary_search, 'glossary' => $glossary ) );
	}

	return json_decode($result, true);
}

function mywpglossary_matching( $content ) {
	if ( is_singular() && in_the_loop() && is_main_query() && !has_block('mywpglossary/glossary') ) {
		$data_glossary = mywpglossary_get_transient_glossary();
		foreach ($data_glossary['glossary_search'] as $key => $value) {
			if(!preg_match('/=".*'.$value.'.*"/i', $content)) {
				$content = preg_replace('/\b('.$value.')\b/imu', $data_glossary['glossary'][$key]['begin'] .'$1' . $data_glossary['glossary'][$key]['end'].' ', $content);
			}
		}
	}
	return $content;

}
add_filter( 'the_content', 'mywpglossary_matching', 10 );



add_action('enqueue_block_editor_assets', 'mywpglossary_block_enqueue', 20);


function mywpglossary_block_enqueue(){
	wp_enqueue_script(
		'mywpglossary-block-checklist', // Unique handle.
		plugins_url( 'js/checklist.min.js', __FILE__ ),
		array('wp-blocks', 'wp-i18n', 'wp-element', 'wp-components', 'wp-editor', 'wp-plugins' , 'wp-edit-post') // Dependencies, defined above.
	);
}
